@@ -178,7 +178,7 @@ gst_theora_dec_class_init (GstTheoraDecClass * klass)
       gst_static_pad_template_get (&theora_dec_src_factory));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&theora_dec_sink_factory));
-  gst_element_class_set_details_simple (element_class,
+  gst_element_class_set_metadata (element_class,
       "Theora video decoder", "Codec/Decoder/Video",
       "decode raw theora streams to raw YUV video",
       "Benjamin Otte <otte@gnome.org>, Wim Taymans <wim@fluendo.com>");
