@@ -23,7 +23,6 @@
  * SECTION:gstaudioencoder
  * @short_description: Base class for audio encoders
  * @see_also: #GstBaseTransform
- * @since: 0.10.36
  *
  * This base class is for audio encoders turning raw audio samples into
  * encoded audio data.
@@ -189,6 +188,8 @@ typedef struct _GstAudioEncoderContext
   GstAudioInfo info;
 
   /* output */
+  GstCaps *caps;
+  gboolean output_caps_changed;
   gint frame_samples_min, frame_samples_max;
   gint frame_max;
   gint lookahead;
@@ -198,6 +199,9 @@ typedef struct _GstAudioEncoderContext
 
   GList *headers;
   gboolean new_headers;
+
+  GstAllocator *allocator;
+  GstAllocationParams params;
 } GstAudioEncoderContext;
 
 struct _GstAudioEncoderPrivate
@@ -229,6 +233,8 @@ struct _GstAudioEncoderPrivate
   gboolean drained;
   /* subclass currently being forcibly drained */
   gboolean force;
+  /* need to handle changed input caps */
+  gboolean do_caps;
 
   /* output bps estimatation */
   /* global in samples seen */
@@ -249,6 +255,7 @@ struct _GstAudioEncoderPrivate
 
   /* pending tags */
   GstTagList *tags;
+  gboolean tags_changed;
   /* pending serialized sink events, will be sent from finish_frame() */
   GList *pending_events;
 };
@@ -325,6 +332,12 @@ static gboolean gst_audio_encoder_sink_query (GstPad * pad, GstObject * parent,
 static GstStateChangeReturn gst_audio_encoder_change_state (GstElement *
     element, GstStateChange transition);
 
+static gboolean gst_audio_encoder_decide_allocation_default (GstAudioEncoder *
+    enc, GstQuery * query);
+static gboolean gst_audio_encoder_propose_allocation_default (GstAudioEncoder *
+    enc, GstQuery * query);
+static gboolean gst_audio_encoder_negotiate_default (GstAudioEncoder * enc);
+
 static void
 gst_audio_encoder_class_init (GstAudioEncoderClass * klass)
 {
@@ -370,6 +383,9 @@ gst_audio_encoder_class_init (GstAudioEncoderClass * klass)
   klass->getcaps = gst_audio_encoder_getcaps_default;
   klass->sink_event = gst_audio_encoder_sink_event_default;
   klass->src_event = gst_audio_encoder_src_event_default;
+  klass->propose_allocation = gst_audio_encoder_propose_allocation_default;
+  klass->decide_allocation = gst_audio_encoder_decide_allocation_default;
+  klass->negotiate = gst_audio_encoder_negotiate_default;
 }
 
 static void
@@ -445,16 +461,22 @@ gst_audio_encoder_reset (GstAudioEncoder * enc, gboolean full)
     enc->priv->ctx.headers = NULL;
     enc->priv->ctx.new_headers = FALSE;
 
+    gst_caps_replace (&enc->priv->ctx.caps, NULL);
     memset (&enc->priv->ctx, 0, sizeof (enc->priv->ctx));
     gst_audio_info_init (&enc->priv->ctx.info);
 
     if (enc->priv->tags)
-      gst_tag_list_free (enc->priv->tags);
+      gst_tag_list_unref (enc->priv->tags);
     enc->priv->tags = NULL;
+    enc->priv->tags_changed = FALSE;
 
     g_list_foreach (enc->priv->pending_events, (GFunc) gst_event_unref, NULL);
     g_list_free (enc->priv->pending_events);
     enc->priv->pending_events = NULL;
+
+    if (enc->priv->ctx.allocator)
+      gst_object_unref (enc->priv->ctx.allocator);
+    enc->priv->ctx.allocator = NULL;
   }
 
   gst_segment_init (&enc->input_segment, GST_FORMAT_TIME);
@@ -568,8 +590,6 @@ gst_audio_encoder_push_event (GstAudioEncoder * enc, GstEvent * event)
  * may be invalidated by a call to this function.
  *
  * Returns: a #GstFlowReturn that should be escalated to caller (of caller)
- *
- * Since: 0.10.36
  */
 GstFlowReturn
 gst_audio_encoder_finish_frame (GstAudioEncoder * enc, GstBuffer * buf,
@@ -589,7 +609,7 @@ gst_audio_encoder_finish_frame (GstAudioEncoder * enc, GstBuffer * buf,
       GST_FLOW_ERROR);
 
   /* subclass should know what it is producing by now */
-  if (!gst_pad_has_current_caps (enc->srcpad))
+  if (!ctx->caps)
     goto no_caps;
 
   GST_AUDIO_ENCODER_STREAM_LOCK (enc);
@@ -598,6 +618,14 @@ gst_audio_encoder_finish_frame (GstAudioEncoder * enc, GstBuffer * buf,
       "accepting %" G_GSIZE_FORMAT " bytes encoded data as %d samples",
       buf ? gst_buffer_get_size (buf) : -1, samples);
 
+  if (G_UNLIKELY (ctx->output_caps_changed
+          || gst_pad_check_reconfigure (enc->srcpad))) {
+    if (!gst_audio_encoder_negotiate (enc)) {
+      ret = GST_FLOW_NOT_NEGOTIATED;
+      goto exit;
+    }
+  }
+
   /* mark subclass still alive and providing */
   if (G_LIKELY (buf))
     priv->got_data = TRUE;
@@ -615,24 +643,26 @@ gst_audio_encoder_finish_frame (GstAudioEncoder * enc, GstBuffer * buf,
   }
 
   /* send after pending events, which likely includes newsegment event */
-  if (G_UNLIKELY (enc->priv->tags)) {
-    GstTagList *tags;
+  if (G_UNLIKELY (enc->priv->tags && enc->priv->tags_changed)) {
 #if 0
     GstCaps *caps;
 #endif
 
     /* add codec info to pending tags */
-    tags = enc->priv->tags;
-    /* no more pending */
-    enc->priv->tags = NULL;
 #if 0
+    if (!enc->priv->tags)
+      enc->priv->tags = gst_tag_list_new ();
+    enc->priv->tags = gst_tag_list_make_writable (enc->priv->tags);
     caps = gst_pad_get_current_caps (enc->srcpad);
-    gst_pb_utils_add_codec_description_to_tag_list (tags, GST_TAG_CODEC, caps);
-    gst_pb_utils_add_codec_description_to_tag_list (tags, GST_TAG_AUDIO_CODEC,
-        caps);
+    gst_pb_utils_add_codec_description_to_tag_list (enc->priv->tags,
+        GST_TAG_CODEC, caps);
+    gst_pb_utils_add_codec_description_to_tag_list (enc->priv->tags,
+        GST_TAG_AUDIO_CODEC, caps);
 #endif
-    GST_DEBUG_OBJECT (enc, "sending tags %" GST_PTR_FORMAT, tags);
-    gst_audio_encoder_push_event (enc, gst_event_new_tag ("GstEncoder", tags));
+    GST_DEBUG_OBJECT (enc, "sending tags %" GST_PTR_FORMAT, enc->priv->tags);
+    gst_audio_encoder_push_event (enc,
+        gst_event_new_tag (gst_tag_list_ref (enc->priv->tags)));
+    enc->priv->tags_changed = FALSE;
   }
 
   /* remove corresponding samples from input */
@@ -1009,6 +1039,18 @@ gst_audio_encoder_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 
   GST_AUDIO_ENCODER_STREAM_LOCK (enc);
 
+  if (G_UNLIKELY (priv->do_caps)) {
+    GstCaps *caps = gst_pad_get_current_caps (enc->sinkpad);
+    if (!caps)
+      goto not_negotiated;
+    if (!gst_audio_encoder_sink_setcaps (enc, caps)) {
+      gst_caps_unref (caps);
+      goto not_negotiated;
+    }
+    gst_caps_unref (caps);
+    priv->do_caps = FALSE;
+  }
+
   /* should know what is coming by now */
   if (!ctx->info.bpf)
     goto not_negotiated;
@@ -1185,8 +1227,8 @@ audio_info_is_equal (GstAudioInfo * from, GstAudioInfo * to)
     return FALSE;
   if (GST_AUDIO_INFO_CHANNELS (from) > 64)
     return TRUE;
-  return memcmp (from->position, to->position,
-      GST_AUDIO_INFO_CHANNELS (from) * sizeof (to->position[0]));
+  return (memcmp (from->position, to->position,
+          GST_AUDIO_INFO_CHANNELS (from) * sizeof (to->position[0])) == 0);
 }
 
 static gboolean
@@ -1291,17 +1333,17 @@ refuse_caps:
  * gst_audio_encoder_proxy_getcaps:
  * @enc: a #GstAudioEncoder
  * @caps: initial caps
+ * @filter: filter caps
  *
  * Returns caps that express @caps (or sink template caps if @caps == NULL)
  * restricted to channel/rate combinations supported by downstream elements
  * (e.g. muxers).
  *
  * Returns: a #GstCaps owned by caller
- *
- * Since: 0.10.36
  */
 GstCaps *
-gst_audio_encoder_proxy_getcaps (GstAudioEncoder * enc, GstCaps * caps)
+gst_audio_encoder_proxy_getcaps (GstAudioEncoder * enc, GstCaps * caps,
+    GstCaps * filter)
 {
   GstCaps *templ_caps = NULL;
   GstCaps *allowed = NULL;
@@ -1353,6 +1395,14 @@ gst_audio_encoder_proxy_getcaps (GstAudioEncoder * enc, GstCaps * caps)
   gst_caps_unref (filter_caps);
   gst_caps_unref (templ_caps);
 
+  if (filter) {
+    GST_LOG_OBJECT (enc, "intersecting with %" GST_PTR_FORMAT, filter);
+    filter_caps = gst_caps_intersect_full (filter, fcaps,
+        GST_CAPS_INTERSECT_FIRST);
+    gst_caps_unref (fcaps);
+    fcaps = filter_caps;
+  }
+
 done:
   gst_caps_replace (&allowed, NULL);
 
@@ -1366,7 +1416,7 @@ gst_audio_encoder_getcaps_default (GstAudioEncoder * enc, GstCaps * filter)
 {
   GstCaps *caps;
 
-  caps = gst_audio_encoder_proxy_getcaps (enc, NULL);
+  caps = gst_audio_encoder_proxy_getcaps (enc, NULL, filter);
   GST_LOG_OBJECT (enc, "returning caps %" GST_PTR_FORMAT, caps);
 
   return caps;
@@ -1445,46 +1495,49 @@ gst_audio_encoder_sink_event_default (GstAudioEncoder * enc, GstEvent * event)
       res = gst_audio_encoder_push_event (enc, event);
       break;
 
-    case GST_EVENT_TAG:
-    {
-      GstTagList *tags;
-
-      gst_event_parse_tag (event, &tags);
-      tags = gst_tag_list_copy (tags);
-      gst_event_unref (event);
-
-      /* FIXME: make generic based on GST_TAG_FLAG_ENCODED */
-      gst_tag_list_remove_tag (tags, GST_TAG_CODEC);
-      gst_tag_list_remove_tag (tags, GST_TAG_AUDIO_CODEC);
-      gst_tag_list_remove_tag (tags, GST_TAG_VIDEO_CODEC);
-      gst_tag_list_remove_tag (tags, GST_TAG_SUBTITLE_CODEC);
-      gst_tag_list_remove_tag (tags, GST_TAG_CONTAINER_FORMAT);
-      gst_tag_list_remove_tag (tags, GST_TAG_BITRATE);
-      gst_tag_list_remove_tag (tags, GST_TAG_NOMINAL_BITRATE);
-      gst_tag_list_remove_tag (tags, GST_TAG_MAXIMUM_BITRATE);
-      gst_tag_list_remove_tag (tags, GST_TAG_MINIMUM_BITRATE);
-      gst_tag_list_remove_tag (tags, GST_TAG_ENCODER);
-      gst_tag_list_remove_tag (tags, GST_TAG_ENCODER_VERSION);
-      event = gst_event_new_tag ("GstEncoder", tags);
-
-      GST_AUDIO_ENCODER_STREAM_LOCK (enc);
-      enc->priv->pending_events =
-          g_list_append (enc->priv->pending_events, event);
-      GST_AUDIO_ENCODER_STREAM_UNLOCK (enc);
-      res = TRUE;
-      break;
-    }
-
     case GST_EVENT_CAPS:
     {
       GstCaps *caps;
 
       gst_event_parse_caps (event, &caps);
-      res = gst_audio_encoder_sink_setcaps (enc, caps);
+      enc->priv->do_caps = TRUE;
+      res = TRUE;
       gst_event_unref (event);
       break;
     }
 
+    case GST_EVENT_TAG:
+    {
+      GstTagList *tags;
+
+      gst_event_parse_tag (event, &tags);
+
+      if (gst_tag_list_get_scope (tags) == GST_TAG_SCOPE_STREAM) {
+        tags = gst_tag_list_copy (tags);
+
+        /* FIXME: make generic based on GST_TAG_FLAG_ENCODED */
+        gst_tag_list_remove_tag (tags, GST_TAG_CODEC);
+        gst_tag_list_remove_tag (tags, GST_TAG_AUDIO_CODEC);
+        gst_tag_list_remove_tag (tags, GST_TAG_VIDEO_CODEC);
+        gst_tag_list_remove_tag (tags, GST_TAG_SUBTITLE_CODEC);
+        gst_tag_list_remove_tag (tags, GST_TAG_CONTAINER_FORMAT);
+        gst_tag_list_remove_tag (tags, GST_TAG_BITRATE);
+        gst_tag_list_remove_tag (tags, GST_TAG_NOMINAL_BITRATE);
+        gst_tag_list_remove_tag (tags, GST_TAG_MAXIMUM_BITRATE);
+        gst_tag_list_remove_tag (tags, GST_TAG_MINIMUM_BITRATE);
+        gst_tag_list_remove_tag (tags, GST_TAG_ENCODER);
+        gst_tag_list_remove_tag (tags, GST_TAG_ENCODER_VERSION);
+
+        gst_audio_encoder_merge_tags (enc, tags, GST_TAG_MERGE_REPLACE);
+        gst_tag_list_unref (tags);
+        gst_event_unref (event);
+        event = NULL;
+        res = TRUE;
+        break;
+      }
+      /* fall through */
+    }
+
     default:
       /* Forward non-serialized events immediately. */
       if (!GST_EVENT_IS_SERIALIZED (event)) {
@@ -1574,6 +1627,14 @@ gst_audio_encoder_sink_query (GstPad * pad, GstObject * parent,
       }
       break;
     }
+    case GST_QUERY_ALLOCATION:
+    {
+      GstAudioEncoderClass *klass = GST_AUDIO_ENCODER_GET_CLASS (enc);
+
+      if (klass->propose_allocation)
+        res = klass->propose_allocation (enc, query);
+      break;
+    }
     default:
       res = gst_pad_query_default (pad, parent, query);
       break;
@@ -1619,6 +1680,43 @@ gst_audio_encoder_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
   return ret;
 }
 
+static gboolean
+gst_audio_encoder_decide_allocation_default (GstAudioEncoder * enc,
+    GstQuery * query)
+{
+  GstAllocator *allocator = NULL;
+  GstAllocationParams params;
+  gboolean update_allocator;
+
+  /* we got configuration from our peer or the decide_allocation method,
+   * parse them */
+  if (gst_query_get_n_allocation_params (query) > 0) {
+    /* try the allocator */
+    gst_query_parse_nth_allocation_param (query, 0, &allocator, &params);
+    update_allocator = TRUE;
+  } else {
+    allocator = NULL;
+    gst_allocation_params_init (&params);
+    update_allocator = FALSE;
+  }
+
+  if (update_allocator)
+    gst_query_set_nth_allocation_param (query, 0, allocator, &params);
+  else
+    gst_query_add_allocation_param (query, allocator, &params);
+  if (allocator)
+    gst_object_unref (allocator);
+
+  return TRUE;
+}
+
+static gboolean
+gst_audio_encoder_propose_allocation_default (GstAudioEncoder * enc,
+    GstQuery * query)
+{
+  return TRUE;
+}
+
 /*
  * gst_audio_encoded_audio_convert:
  * @fmt: audio format of the encoded audio
@@ -1633,8 +1731,6 @@ gst_audio_encoder_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
  * @dest_format for encoded audio data.  Conversion is possible between
  * BYTE and TIME format by using estimated bitrate based on
  * @samples and @bytes (and @fmt).
- *
- * Since: 0.10.36
  */
 /* FIXME: make gst_audio_encoded_audio_convert() public? */
 static gboolean
@@ -1860,7 +1956,7 @@ static gboolean
 gst_audio_encoder_activate (GstAudioEncoder * enc, gboolean active)
 {
   GstAudioEncoderClass *klass;
-  gboolean result = FALSE;
+  gboolean result = TRUE;
 
   klass = GST_AUDIO_ENCODER_GET_CLASS (enc);
 
@@ -1871,8 +1967,9 @@ gst_audio_encoder_activate (GstAudioEncoder * enc, gboolean active)
   if (active) {
 
     if (enc->priv->tags)
-      gst_tag_list_free (enc->priv->tags);
+      gst_tag_list_unref (enc->priv->tags);
     enc->priv->tags = gst_tag_list_new_empty ();
+    enc->priv->tags_changed = FALSE;
 
     if (!enc->priv->active && klass->start)
       result = klass->start (enc);
@@ -1919,8 +2016,6 @@ gst_audio_encoder_sink_activate_mode (GstPad * pad, GstObject * parent,
  * @enc: a #GstAudioEncoder
  *
  * Returns: a #GstAudioInfo describing the input audio format
- *
- * Since: 0.10.36
  */
 GstAudioInfo *
 gst_audio_encoder_get_audio_info (GstAudioEncoder * enc)
@@ -1940,8 +2035,6 @@ gst_audio_encoder_get_audio_info (GstAudioEncoder * enc)
  *
  * If an exact number of samples is required, gst_audio_encoder_set_frame_samples_max()
  * must be called with the same number.
- *
- * Since: 0.10.36
  */
 void
 gst_audio_encoder_set_frame_samples_min (GstAudioEncoder * enc, gint num)
@@ -1956,8 +2049,6 @@ gst_audio_encoder_set_frame_samples_min (GstAudioEncoder * enc, gint num)
  * @enc: a #GstAudioEncoder
  *
  * Returns: currently minimum requested samples per frame
- *
- * Since: 0.10.36
  */
 gint
 gst_audio_encoder_get_frame_samples_min (GstAudioEncoder * enc)
@@ -1977,8 +2068,6 @@ gst_audio_encoder_get_frame_samples_min (GstAudioEncoder * enc)
  *
  * If an exact number of samples is required, gst_audio_encoder_set_frame_samples_min()
  * must be called with the same number.
- *
- * Since: 0.10.36
  */
 void
 gst_audio_encoder_set_frame_samples_max (GstAudioEncoder * enc, gint num)
@@ -1989,12 +2078,10 @@ gst_audio_encoder_set_frame_samples_max (GstAudioEncoder * enc, gint num)
 }
 
 /**
- * gst_audio_encoder_get_frame_samples_min:
+ * gst_audio_encoder_get_frame_samples_max:
  * @enc: a #GstAudioEncoder
  *
  * Returns: currently maximum requested samples per frame
- *
- * Since: 0.10.36
  */
 gint
 gst_audio_encoder_get_frame_samples_max (GstAudioEncoder * enc)
@@ -2011,8 +2098,6 @@ gst_audio_encoder_get_frame_samples_max (GstAudioEncoder * enc)
  *
  * Sets max number of frames accepted at once (assumed minimally 1).
  * Requires @frame_samples_min and @frame_samples_max to be the equal.
- *
- * Since: 0.10.36
  */
 void
 gst_audio_encoder_set_frame_max (GstAudioEncoder * enc, gint num)
@@ -2027,8 +2112,6 @@ gst_audio_encoder_set_frame_max (GstAudioEncoder * enc, gint num)
  * @enc: a #GstAudioEncoder
  *
  * Returns: currently configured maximum handled frames
- *
- * Since: 0.10.36
  */
 gint
 gst_audio_encoder_get_frame_max (GstAudioEncoder * enc)
@@ -2044,8 +2127,6 @@ gst_audio_encoder_get_frame_max (GstAudioEncoder * enc)
  * @num: lookahead
  *
  * Sets encoder lookahead (in units of input rate samples)
- *
- * Since: 0.10.36
  */
 void
 gst_audio_encoder_set_lookahead (GstAudioEncoder * enc, gint num)
@@ -2076,8 +2157,6 @@ gst_audio_encoder_get_lookahead (GstAudioEncoder * enc)
  * @max: maximum latency
  *
  * Sets encoder latency.
- *
- * Since: 0.10.36
  */
 void
 gst_audio_encoder_set_latency (GstAudioEncoder * enc,
@@ -2099,8 +2178,6 @@ gst_audio_encoder_set_latency (GstAudioEncoder * enc,
  *
  * Sets the variables pointed to by @min and @max to the currently configured
  * latency.
- *
- * Since: 0.10.36
  */
 void
 gst_audio_encoder_get_latency (GstAudioEncoder * enc,
@@ -2118,8 +2195,9 @@ gst_audio_encoder_get_latency (GstAudioEncoder * enc,
 
 /**
  * gst_audio_encoder_set_headers:
- * @encoder: a #GstAudioEncoder
- * @headers: (transfer full): a list of #GstBuffer containing the codec header
+ * @enc: a #GstAudioEncoder
+ * @headers: (transfer full) (element-type Gst.Buffer): a list of
+ *   #GstBuffer containing the codec header
  *
  * Set the codec headers to be sent downstream whenever requested.
  */
@@ -2144,8 +2222,6 @@ gst_audio_encoder_set_headers (GstAudioEncoder * enc, GList * headers)
  * Enable or disable encoder granule handling.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
 gst_audio_encoder_set_mark_granule (GstAudioEncoder * enc, gboolean enabled)
@@ -2168,8 +2244,6 @@ gst_audio_encoder_set_mark_granule (GstAudioEncoder * enc, gboolean enabled)
  * Returns: TRUE if granule marking is enabled.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 gboolean
 gst_audio_encoder_get_mark_granule (GstAudioEncoder * enc)
@@ -2193,8 +2267,6 @@ gst_audio_encoder_get_mark_granule (GstAudioEncoder * enc)
  * Enable or disable encoder perfect output timestamp preference.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
 gst_audio_encoder_set_perfect_timestamp (GstAudioEncoder * enc,
@@ -2218,8 +2290,6 @@ gst_audio_encoder_set_perfect_timestamp (GstAudioEncoder * enc,
  * Returns: TRUE if perfect timestamp setting enabled.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 gboolean
 gst_audio_encoder_get_perfect_timestamp (GstAudioEncoder * enc)
@@ -2243,8 +2313,6 @@ gst_audio_encoder_get_perfect_timestamp (GstAudioEncoder * enc)
  * Sets encoder hard resync handling.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
 gst_audio_encoder_set_hard_resync (GstAudioEncoder * enc, gboolean enabled)
@@ -2267,8 +2335,6 @@ gst_audio_encoder_set_hard_resync (GstAudioEncoder * enc, gboolean enabled)
  * Returns: TRUE if hard resync is enabled.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 gboolean
 gst_audio_encoder_get_hard_resync (GstAudioEncoder * enc)
@@ -2292,11 +2358,9 @@ gst_audio_encoder_get_hard_resync (GstAudioEncoder * enc)
  * Configures encoder audio jitter tolerance threshold.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
-gst_audio_encoder_set_tolerance (GstAudioEncoder * enc, gint64 tolerance)
+gst_audio_encoder_set_tolerance (GstAudioEncoder * enc, GstClockTime tolerance)
 {
   g_return_if_fail (GST_IS_AUDIO_ENCODER (enc));
 
@@ -2314,13 +2378,11 @@ gst_audio_encoder_set_tolerance (GstAudioEncoder * enc, gint64 tolerance)
  * Returns: encoder audio jitter tolerance threshold.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
-gint64
+GstClockTime
 gst_audio_encoder_get_tolerance (GstAudioEncoder * enc)
 {
-  gint64 result;
+  GstClockTime result;
 
   g_return_val_if_fail (GST_IS_AUDIO_ENCODER (enc), 0);
 
@@ -2342,8 +2404,6 @@ gst_audio_encoder_get_tolerance (GstAudioEncoder * enc)
  * will simply be discarded.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
 gst_audio_encoder_set_hard_min (GstAudioEncoder * enc, gboolean enabled)
@@ -2364,8 +2424,6 @@ gst_audio_encoder_set_hard_min (GstAudioEncoder * enc, gboolean enabled)
  * Returns: TRUE if hard minimum handling is enabled.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 gboolean
 gst_audio_encoder_get_hard_min (GstAudioEncoder * enc)
@@ -2392,8 +2450,6 @@ gst_audio_encoder_get_hard_min (GstAudioEncoder * enc)
  * real data.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
 gst_audio_encoder_set_drainable (GstAudioEncoder * enc, gboolean enabled)
@@ -2414,8 +2470,6 @@ gst_audio_encoder_set_drainable (GstAudioEncoder * enc, gboolean enabled)
  * Returns: TRUE if drainable handling is enabled.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 gboolean
 gst_audio_encoder_get_drainable (GstAudioEncoder * enc)
@@ -2446,8 +2500,6 @@ gst_audio_encoder_get_drainable (GstAudioEncoder * enc)
  * of the usual CODEC/AUDIO_CODEC tags.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
 gst_audio_encoder_merge_tags (GstAudioEncoder * enc,
@@ -2458,14 +2510,107 @@ gst_audio_encoder_merge_tags (GstAudioEncoder * enc,
   g_return_if_fail (GST_IS_AUDIO_ENCODER (enc));
   g_return_if_fail (tags == NULL || GST_IS_TAG_LIST (tags));
 
-  GST_OBJECT_LOCK (enc);
+  GST_AUDIO_ENCODER_STREAM_LOCK (enc);
   if (tags)
     GST_DEBUG_OBJECT (enc, "merging tags %" GST_PTR_FORMAT, tags);
   otags = enc->priv->tags;
   enc->priv->tags = gst_tag_list_merge (enc->priv->tags, tags, mode);
   if (otags)
-    gst_tag_list_free (otags);
-  GST_OBJECT_UNLOCK (enc);
+    gst_tag_list_unref (otags);
+  enc->priv->tags_changed = TRUE;
+  GST_AUDIO_ENCODER_STREAM_UNLOCK (enc);
+}
+
+static gboolean
+gst_audio_encoder_negotiate_default (GstAudioEncoder * enc)
+{
+  GstAudioEncoderClass *klass;
+  gboolean res = FALSE;
+  GstQuery *query = NULL;
+  GstAllocator *allocator;
+  GstAllocationParams params;
+  GstCaps *caps;
+
+  g_return_val_if_fail (GST_IS_AUDIO_ENCODER (enc), FALSE);
+  g_return_val_if_fail (GST_IS_CAPS (enc->priv->ctx.caps), FALSE);
+
+  klass = GST_AUDIO_ENCODER_GET_CLASS (enc);
+
+  caps = enc->priv->ctx.caps;
+
+  GST_DEBUG_OBJECT (enc, "Setting srcpad caps %" GST_PTR_FORMAT, caps);
+
+  res = gst_pad_set_caps (enc->srcpad, caps);
+  if (!res)
+    goto done;
+  enc->priv->ctx.output_caps_changed = FALSE;
+
+  query = gst_query_new_allocation (caps, TRUE);
+  if (!gst_pad_peer_query (enc->srcpad, query)) {
+    GST_DEBUG_OBJECT (enc, "didn't get downstream ALLOCATION hints");
+  }
+
+  g_assert (klass->decide_allocation != NULL);
+  res = klass->decide_allocation (enc, query);
+
+  GST_DEBUG_OBJECT (enc, "ALLOCATION (%d) params: %" GST_PTR_FORMAT, res,
+      query);
+
+  if (!res)
+    goto no_decide_allocation;
+
+  /* we got configuration from our peer or the decide_allocation method,
+   * parse them */
+  if (gst_query_get_n_allocation_params (query) > 0) {
+    gst_query_parse_nth_allocation_param (query, 0, &allocator, &params);
+  } else {
+    allocator = NULL;
+    gst_allocation_params_init (&params);
+  }
+
+  if (enc->priv->ctx.allocator)
+    gst_object_unref (enc->priv->ctx.allocator);
+  enc->priv->ctx.allocator = allocator;
+  enc->priv->ctx.params = params;
+
+done:
+  if (query)
+    gst_query_unref (query);
+
+  return res;
+
+  /* ERRORS */
+no_decide_allocation:
+  {
+    GST_WARNING_OBJECT (enc, "Subclass failed to decide allocation");
+    goto done;
+  }
+}
+
+/**
+ * gst_audio_encoder_negotiate:
+ * @enc: a #GstAudioEncoder
+ *
+ * Negotiate with downstreame elements to currently configured #GstCaps.
+ *
+ * Returns: #TRUE if the negotiation succeeded, else #FALSE.
+ */
+gboolean
+gst_audio_encoder_negotiate (GstAudioEncoder * enc)
+{
+  GstAudioEncoderClass *klass;
+  gboolean ret = TRUE;
+
+  g_return_val_if_fail (GST_IS_AUDIO_ENCODER (enc), FALSE);
+
+  klass = GST_AUDIO_ENCODER_GET_CLASS (enc);
+
+  GST_AUDIO_ENCODER_STREAM_LOCK (enc);
+  if (klass->negotiate)
+    ret = klass->negotiate (enc);
+  GST_AUDIO_ENCODER_STREAM_UNLOCK (enc);
+
+  return ret;
 }
 
 /*
@@ -2480,11 +2625,12 @@ gst_audio_encoder_merge_tags (GstAudioEncoder * enc,
 gboolean
 gst_audio_encoder_set_output_format (GstAudioEncoder * enc, GstCaps * caps)
 {
-  gboolean res = FALSE;
+  gboolean res = TRUE;
   GstCaps *templ_caps;
 
   GST_DEBUG_OBJECT (enc, "Setting srcpad caps %" GST_PTR_FORMAT, caps);
 
+  GST_AUDIO_ENCODER_STREAM_LOCK (enc);
   if (!gst_caps_is_fixed (caps))
     goto refuse_caps;
 
@@ -2496,9 +2642,12 @@ gst_audio_encoder_set_output_format (GstAudioEncoder * enc, GstCaps * caps)
   }
   gst_caps_unref (templ_caps);
 
-  res = gst_pad_set_caps (enc->srcpad, caps);
+  gst_caps_replace (&enc->priv->ctx.caps, caps);
+  enc->priv->ctx.output_caps_changed = TRUE;
 
 done:
+  GST_AUDIO_ENCODER_STREAM_UNLOCK (enc);
+
   return res;
 
   /* ERRORS */
@@ -2509,3 +2658,67 @@ refuse_caps:
     goto done;
   }
 }
+
+/**
+ * gst_audio_encoder_allocate_output_buffer:
+ * @enc: a #GstAudioEncoder
+ * @size: size of the buffer
+ *
+ * Helper function that allocates a buffer to hold an encoded audio frame
+ * for @enc's current output format.
+ *
+ * Returns: (transfer full): allocated buffer
+ */
+GstBuffer *
+gst_audio_encoder_allocate_output_buffer (GstAudioEncoder * enc, gsize size)
+{
+  GstBuffer *buffer = NULL;
+
+  g_return_val_if_fail (size > 0, NULL);
+
+  GST_DEBUG ("alloc src buffer");
+
+  GST_AUDIO_ENCODER_STREAM_LOCK (enc);
+
+  if (G_UNLIKELY (enc->priv->ctx.output_caps_changed || (enc->priv->ctx.caps
+              && gst_pad_check_reconfigure (enc->srcpad)))) {
+    if (!gst_audio_encoder_negotiate (enc))
+      goto done;
+  }
+
+  buffer =
+      gst_buffer_new_allocate (enc->priv->ctx.allocator, size,
+      &enc->priv->ctx.params);
+
+done:
+  GST_AUDIO_ENCODER_STREAM_UNLOCK (enc);
+
+  return buffer;
+}
+
+/**
+ * gst_audio_encoder_get_allocator:
+ * @enc: a #GstAudioEncoder
+ * @allocator: (out) (allow-none) (transfer full): the #GstAllocator
+ * used
+ * @params: (out) (allow-none) (transfer full): the
+ * #GstAllocatorParams of @allocator
+ *
+ * Lets #GstAudioEncoder sub-classes to know the memory @allocator
+ * used by the base class and its @params.
+ *
+ * Unref the @allocator after use it.
+ */
+void
+gst_audio_encoder_get_allocator (GstAudioEncoder * enc,
+    GstAllocator ** allocator, GstAllocationParams * params)
+{
+  g_return_if_fail (GST_IS_AUDIO_ENCODER (enc));
+
+  if (allocator)
+    *allocator = enc->priv->ctx.allocator ?
+        gst_object_ref (enc->priv->ctx.allocator) : NULL;
+
+  if (params)
+    *params = enc->priv->ctx.params;
+}
