@@ -65,11 +65,9 @@ typedef struct
 
 static const FormatInfo formats[] = {
   /* container/tag formats with static descriptions */
-  {"audio/annodex", "Ogg", FLAG_CONTAINER},
   {"audio/ogg", "Ogg", FLAG_CONTAINER},
   {"audio/webm", "WebM", FLAG_CONTAINER},
   {"audio/x-matroska", "Matroska", FLAG_CONTAINER},
-  {"application/annodex", "Ogg", FLAG_CONTAINER},
   {"application/gxf", "General Exchange Format (GXF)", FLAG_CONTAINER},
   {"application/ogg", "Ogg", FLAG_CONTAINER},
   {"application/kate", "Ogg", FLAG_CONTAINER},
@@ -83,7 +81,6 @@ static const FormatInfo formats[] = {
   {"application/x-pn-realaudio", "RealAudio", FLAG_CONTAINER},
   {"application/x-yuv4mpeg", "Y4M", FLAG_CONTAINER},
   {"multipart/x-mixed-replace", "Multipart", FLAG_CONTAINER},
-  {"video/annodex", "Ogg", FLAG_CONTAINER},
   {"video/ogg", "Ogg", FLAG_CONTAINER},
   {"video/x-fli", "FLI/FLC/FLX Animation", FLAG_CONTAINER},
   {"video/x-flv", "Flash", FLAG_CONTAINER},
@@ -120,6 +117,7 @@ static const FormatInfo formats[] = {
   {"audio/x-dts", "DTS", 0},
   {"audio/x-private1-dts", "DTS", 0},
   {"audio/x-dv", "DV Audio", 0},
+  {"audio/x-eac3", "E-AC-3 (ATSC A/52B)", 0},
   {"audio/x-flac", N_("Free Lossless Audio Codec (FLAC)"), 0},
   {"audio/x-gsm", "GSM", 0},
   {"audio/x-iec958", "S/PDIF IEC958", 0},       /* TODO: check description */
@@ -162,7 +160,6 @@ static const FormatInfo formats[] = {
   /* video formats with static descriptions */
   {"video/sp5x", "Sunplus JPEG 5.x", 0},
   {"video/vivo", "Vivo", 0},
-  {"video/x-3ivx", "3ivx", 0},
   {"video/x-4xm", "4X Technologies Video", 0},
   {"video/x-apple-video", "Apple video", 0},
   {"video/x-aasc", "Autodesk Animator", 0},
@@ -173,8 +170,7 @@ static const FormatInfo formats[] = {
   {"video/x-compressed-yuv", N_("CYUV Lossless"), 0},
   {"video/x-dirac", "Dirac", 0},
   {"video/x-dnxhd", "Digital Nonlinear Extensible High Definition (DNxHD)", 0},
-  /* FIXME 0.11: rename to subpicture/x-dvd or so */
-  {"video/x-dvd-subpicture", "DVD subpicture", 0},
+  {"subpicture/x-dvd", "DVD subpicture", 0},
   {"video/x-ffv", N_("FFMpeg v1"), 0},
   {"video/x-flash-screen", "Flash Screen Video", 0},
   {"video/x-flash-video", "Sorenson Spark Video", 0},
@@ -210,7 +206,6 @@ static const FormatInfo formats[] = {
   {"video/x-vp6-alpha", "On2 VP6 with alpha", 0},
   {"video/x-vp7", "On2 VP7", 0},
   {"video/x-vp8", "VP8", 0},
-  {"video/x-xvid", "XVID MPEG-4", 0},
   {"video/x-zlib", "Lossless zlib video", 0},
   {"video/x-zmbv", "Zip Motion Block video", 0},
 
@@ -256,6 +251,7 @@ static const FormatInfo formats[] = {
   {"application/x-kate", "Kate", 0},
   {"subtitle/x-kate", N_("Kate subtitle format"), 0},
   {"subpicture/x-dvb", "DVB subtitles", 0},
+  {"subpicture/x-pgs", "PGS subtitles", 0},
 
   /* non-audio/video/container formats */
   {"hdv/aux-v", "HDV AUX-V", 0},
