@@ -91,12 +91,16 @@
 #include "struct_arm.h"
 #define HAVE_ABI_SIZES TRUE
 #else
-/* in case someone wants to generate a new arch */
+#ifdef __powerpc__
+#include "struct_ppc32.h"
+#define HAVE_ABI_SIZES TRUE
+#else /* in case someone wants to generate a new arch */
 #include "struct_i386.h"
 #define HAVE_ABI_SIZES FALSE
 #endif
 #endif
 #endif
+#endif
 
 GST_START_TEST (test_ABI)
 {
