@@ -204,7 +204,7 @@ GST_START_TEST (test_musicbrainz_tag_registration)
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_MUSICBRAINZ_TRMID, "5678901");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_ARTIST_SORTNAME, "Five, 678901");
 
-  gst_tag_list_free (list);
+  gst_tag_list_unref (list);
 }
 
 GST_END_TEST;
@@ -276,15 +276,16 @@ GST_START_TEST (test_vorbis_tags)
   ASSERT_TAG_LIST_HAS_UINT (list, GST_TAG_ALBUM_VOLUME_COUNT, 2);
 
   {
-    GDate *date = NULL;
+    GstDateTime *dt = NULL;
 
-    fail_unless (gst_tag_list_get_date (list, GST_TAG_DATE, &date));
-    fail_unless (date != NULL);
-    fail_unless (g_date_get_day (date) == 31);
-    fail_unless (g_date_get_month (date) == G_DATE_DECEMBER);
-    fail_unless (g_date_get_year (date) == 1954);
+    fail_unless (gst_tag_list_get_date_time (list, GST_TAG_DATE_TIME, &dt));
+    fail_unless (dt != NULL);
+    fail_unless (gst_date_time_get_day (dt) == 31);
+    fail_unless (gst_date_time_get_month (dt) == 12);
+    fail_unless (gst_date_time_get_year (dt) == 1954);
+    fail_unless (!gst_date_time_has_time (dt));
 
-    g_date_free (date);
+    gst_date_time_unref (dt);
   }
 
   /* unknown vorbis comments should go into a GST_TAG_EXTENDED_COMMENT */
@@ -341,45 +342,45 @@ GST_START_TEST (test_vorbis_tags)
     GST_LOG ("even_newer_list = %" GST_PTR_FORMAT, even_newer_list);
     fail_unless (gst_tag_list_is_equal (new_list, even_newer_list));
 
-    gst_tag_list_free (new_list);
-    gst_tag_list_free (even_newer_list);
+    gst_tag_list_unref (new_list);
+    gst_tag_list_unref (even_newer_list);
     gst_buffer_unref (buf);
     gst_buffer_unref (buf2);
   }
 
   /* there can only be one language per taglist ... */
-  gst_tag_list_free (list);
+  gst_tag_list_unref (list);
   list = gst_tag_list_new_empty ();
   gst_vorbis_tag_add (list, "LANGUAGE", "fr");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_LANGUAGE_CODE, "fr");
 
-  gst_tag_list_free (list);
+  gst_tag_list_unref (list);
   list = gst_tag_list_new_empty ();
   gst_vorbis_tag_add (list, "LANGUAGE", "[fr]");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_LANGUAGE_CODE, "fr");
 
-  gst_tag_list_free (list);
+  gst_tag_list_unref (list);
   list = gst_tag_list_new_empty ();
   gst_vorbis_tag_add (list, "LANGUAGE", "French [fr]");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_LANGUAGE_CODE, "fr");
 
-  gst_tag_list_free (list);
+  gst_tag_list_unref (list);
   list = gst_tag_list_new_empty ();
   gst_vorbis_tag_add (list, "LANGUAGE", "[eng] English");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_LANGUAGE_CODE, "eng");
 
-  gst_tag_list_free (list);
+  gst_tag_list_unref (list);
   list = gst_tag_list_new_empty ();
   gst_vorbis_tag_add (list, "LANGUAGE", "eng");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_LANGUAGE_CODE, "eng");
 
-  gst_tag_list_free (list);
+  gst_tag_list_unref (list);
   list = gst_tag_list_new_empty ();
   gst_vorbis_tag_add (list, "LANGUAGE", "[eng]");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_LANGUAGE_CODE, "eng");
 
   /* free-form *sigh* */
-  gst_tag_list_free (list);
+  gst_tag_list_unref (list);
   list = gst_tag_list_new_empty ();
   gst_vorbis_tag_add (list, "LANGUAGE", "English");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_LANGUAGE_CODE, "English");
@@ -411,7 +412,7 @@ GST_START_TEST (test_vorbis_tags)
     gst_buffer_unref (buf2);
   }
 
-  gst_tag_list_free (list);
+  gst_tag_list_unref (list);
 
   /* make sure gst_tag_list_from_vorbiscomment_buffer() works with an
    * empty ID (for Speex) */
@@ -447,13 +448,13 @@ GST_START_TEST (test_vorbis_tags)
     fail_unless (list != NULL);
     fail_unless (gst_tag_list_n_tags (list) == 0);
     g_free (vendor);
-    gst_tag_list_free (list);
+    gst_tag_list_unref (list);
 
     /* now again without vendor */
     list = gst_tag_list_from_vorbiscomment_buffer (buf, NULL, 0, NULL);
     fail_unless (list != NULL);
     fail_unless (gst_tag_list_n_tags (list) == 0);
-    gst_tag_list_free (list);
+    gst_tag_list_unref (list);
 
     gst_buffer_unref (buf);
   }
@@ -495,7 +496,7 @@ GST_START_TEST (test_vorbis_tags)
     fail_unless (gst_tag_list_n_tags (list) == 1);
     ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_ARTIST, "foo bar");
     g_free (vendor);
-    gst_tag_list_free (list);
+    gst_tag_list_unref (list);
 
     /* now again without vendor */
     list = gst_tag_list_from_vorbiscomment_buffer (buf,
@@ -503,57 +504,63 @@ GST_START_TEST (test_vorbis_tags)
     fail_unless (list != NULL);
     fail_unless (gst_tag_list_n_tags (list) == 1);
     ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_ARTIST, "foo bar");
-    gst_tag_list_free (list);
+    gst_tag_list_unref (list);
 
     gst_buffer_unref (buf);
   }
 
   /* check date with time */
   {
-    GDate *date = NULL;
+    GstDateTime *dt = NULL;
 
     list = gst_tag_list_new_empty ();
     gst_vorbis_tag_add (list, "DATE", "2006-09-25 22:02:38");
 
-    fail_unless (gst_tag_list_get_date_index (list, GST_TAG_DATE, 0, &date));
-    fail_unless (date != NULL);
-    fail_unless (g_date_get_day (date) == 25);
-    fail_unless (g_date_get_month (date) == G_DATE_SEPTEMBER);
-    fail_unless (g_date_get_year (date) == 2006);
+    fail_unless (gst_tag_list_get_date_time (list, GST_TAG_DATE_TIME, &dt));
+    fail_unless (dt != NULL);
+    fail_unless (gst_date_time_get_day (dt) == 25);
+    fail_unless (gst_date_time_get_month (dt) == 9);
+    fail_unless (gst_date_time_get_year (dt) == 2006);
+    fail_unless (gst_date_time_has_time (dt));
 
-    g_date_free (date);
-    gst_tag_list_free (list);
+    gst_date_time_unref (dt);
+    gst_tag_list_unref (list);
   }
 
   /* check date with month/day of 00-00 */
   {
-    GDate *date = NULL;
+    GstDateTime *dt = NULL;
 
     list = gst_tag_list_new_empty ();
     gst_vorbis_tag_add (list, "DATE", "1992-00-00");
 
-    fail_unless (gst_tag_list_get_date_index (list, GST_TAG_DATE, 0, &date));
-    fail_unless (date != NULL);
-    fail_unless (g_date_get_year (date) == 1992);
+    fail_unless (gst_tag_list_get_date_time (list, GST_TAG_DATE_TIME, &dt));
+    fail_unless (dt != NULL);
+    fail_unless (gst_date_time_get_year (dt) == 1992);
+    fail_unless (!gst_date_time_has_month (dt));
+    fail_unless (!gst_date_time_has_day (dt));
+    fail_unless (!gst_date_time_has_time (dt));
 
-    g_date_free (date);
-    gst_tag_list_free (list);
+    gst_date_time_unref (dt);
+    gst_tag_list_unref (list);
   }
 
   /* check date with valid month, but day of 00 */
   {
-    GDate *date = NULL;
+    GstDateTime *dt = NULL;
 
     list = gst_tag_list_new_empty ();
     gst_vorbis_tag_add (list, "DATE", "1992-05-00");
 
-    fail_unless (gst_tag_list_get_date_index (list, GST_TAG_DATE, 0, &date));
-    fail_unless (date != NULL);
-    fail_unless (g_date_get_year (date) == 1992);
-    fail_unless (g_date_get_month (date) == G_DATE_MAY);
+    fail_unless (gst_tag_list_get_date_time (list, GST_TAG_DATE_TIME, &dt));
+    fail_unless (dt != NULL);
+    fail_unless (gst_date_time_get_year (dt) == 1992);
+    fail_unless (gst_date_time_get_month (dt) == 5);
+    fail_unless (!gst_date_time_has_day (dt));
+    fail_unless (!gst_date_time_has_time (dt));
 
-    g_date_free (date);
-    gst_tag_list_free (list);
+    gst_date_time_unref (dt);
+    gst_tag_list_unref (list);
   }
 }
 
@@ -631,8 +638,8 @@ GST_START_TEST (test_id3v1_utf8_tag)
     /* genre */
     0x11
   };
+  GstDateTime *dt;
   GstTagList *tags;
-  GDate *d;
   gchar *s;
 
   /* set this, to make sure UTF-8 strings are really interpreted properly
@@ -662,14 +669,17 @@ GST_START_TEST (test_id3v1_utf8_tag)
   fail_unless_equals_string (s, "Best of (Prostě úžasný)");
   g_free (s);
 
-  d = NULL;
-  fail_unless (gst_tag_list_get_date (tags, GST_TAG_DATE, &d));
-  fail_unless (d != NULL);
-  fail_unless_equals_int (g_date_get_year (d), 2000);
-  g_date_free (d);
-  d = NULL;
+  dt = NULL;
+  fail_unless (gst_tag_list_get_date_time (tags, GST_TAG_DATE_TIME, &dt));
+  fail_unless (dt != NULL);
+  fail_unless_equals_int (gst_date_time_get_year (dt), 2000);
+  fail_if (gst_date_time_has_month (dt));
+  fail_if (gst_date_time_has_day (dt));
+  fail_if (gst_date_time_has_time (dt));
+  gst_date_time_unref (dt);
+  dt = NULL;
 
-  gst_tag_list_free (tags);
+  gst_tag_list_unref (tags);
 
   g_unsetenv ("GST_ID3V1_TAG_ENCODING");
 }
@@ -952,7 +962,7 @@ GST_START_TEST (test_xmp_formatting)
       GST_TAG_DESCRIPTION, "test decription",
       GST_TAG_KEYWORDS, "keyword1", GST_TAG_KEYWORDS, "keyword2", NULL);
 
-  buf = gst_tag_list_to_xmp_buffer (list, FALSE);
+  buf = gst_tag_list_to_xmp_buffer (list, FALSE, NULL);
   fail_unless (buf != NULL);
 
   gst_buffer_map (buf, &map, GST_MAP_READ);
@@ -969,7 +979,7 @@ GST_START_TEST (test_xmp_formatting)
   gst_buffer_unmap (buf, &map);
 
   gst_buffer_unref (buf);
-  gst_tag_list_free (list);
+  gst_tag_list_unref (list);
 }
 
 GST_END_TEST;
@@ -1046,7 +1056,7 @@ GST_START_TEST (test_xmp_parsing)
         }
       }
       if (list)
-        gst_tag_list_free (list);
+        gst_tag_list_unref (list);
 
       gst_buffer_unref (buf);
       g_free (text);
@@ -1065,13 +1075,13 @@ do_xmp_tag_serialization_deserialization (GstTagList * taglist,
   GstTagList *taglist2;
   GstBuffer *buf;
 
-  buf = gst_tag_list_to_xmp_buffer_full (taglist, TRUE, schemas);
+  buf = gst_tag_list_to_xmp_buffer (taglist, TRUE, schemas);
   taglist2 = gst_tag_list_from_xmp_buffer (buf);
 
   fail_unless (gst_tag_list_is_equal (taglist, taglist2));
 
   gst_buffer_unref (buf);
-  gst_tag_list_free (taglist2);
+  gst_tag_list_unref (taglist2);
 }
 
 static void
@@ -1083,13 +1093,12 @@ do_simple_xmp_tag_serialization_deserialization (const gchar * gsttag,
   gst_tag_list_add_value (taglist, GST_TAG_MERGE_REPLACE, gsttag, value);
 
   do_xmp_tag_serialization_deserialization (taglist, NULL);
-  gst_tag_list_free (taglist);
+  gst_tag_list_unref (taglist);
 }
 
 GST_START_TEST (test_xmp_tags_serialization_deserialization)
 {
   GValue value = { 0 };
-  GDate *date;
   GstDateTime *datetime;
 
   gst_tag_register_musicbrainz_tags ();
@@ -1216,11 +1225,10 @@ GST_START_TEST (test_xmp_tags_serialization_deserialization)
       (GST_TAG_CAPTURING_EXPOSURE_COMPENSATION, &value);
   g_value_unset (&value);
 
-  g_value_init (&value, G_TYPE_DATE);
-  date = g_date_new_dmy (22, 3, 2010);
-  g_value_set_boxed (&value, date);
-  g_date_free (date);
-  do_simple_xmp_tag_serialization_deserialization (GST_TAG_DATE, &value);
+  g_value_init (&value, GST_TYPE_DATE_TIME);
+  datetime = gst_date_time_new_ymd (2010, 3, 22);
+  g_value_take_boxed (&value, datetime);
+  do_simple_xmp_tag_serialization_deserialization (GST_TAG_DATE_TIME, &value);
   g_value_unset (&value);
 
   g_value_init (&value, G_TYPE_UINT);
@@ -1276,13 +1284,13 @@ GST_START_TEST (test_xmp_compound_tags)
       GST_TAG_KEYWORDS, "k2", GST_TAG_TITLE, "title", GST_TAG_KEYWORDS, "k3",
       NULL);
   do_xmp_tag_serialization_deserialization (taglist, NULL);
-  gst_tag_list_free (taglist);
+  gst_tag_list_unref (taglist);
 
   taglist = gst_tag_list_new_empty ();
   gst_tag_list_add (taglist, GST_TAG_MERGE_APPEND, GST_TAG_GEO_LOCATION_COUNTRY,
       "Brazil", GST_TAG_GEO_LOCATION_CITY, "Campina Grande", NULL);
   do_xmp_tag_serialization_deserialization (taglist, schemas);
-  gst_tag_list_free (taglist);
+  gst_tag_list_unref (taglist);
 }
 
 GST_END_TEST;
@@ -1325,7 +1333,7 @@ GST_START_TEST (test_exif_parsing)
   gst_tag_list_peek_string_index (taglist, GST_TAG_COPYRIGHT, 0, &str);
   fail_unless_equals_string (str, "my copyright");
 
-  gst_tag_list_free (taglist);
+  gst_tag_list_unref (taglist);
   gst_buffer_unref (buf);
 }
 
@@ -1344,7 +1352,7 @@ do_exif_tag_serialization_deserialization (GstTagList * taglist)
   gst_buffer_unref (buf);
 
   fail_unless (gst_tag_list_is_equal (taglist, taglist2));
-  gst_tag_list_free (taglist2);
+  gst_tag_list_unref (taglist2);
 
   /* BE */
   buf = gst_tag_list_to_exif_buffer (taglist, G_BIG_ENDIAN, 0);
@@ -1352,7 +1360,7 @@ do_exif_tag_serialization_deserialization (GstTagList * taglist)
   gst_buffer_unref (buf);
 
   fail_unless (gst_tag_list_is_equal (taglist, taglist2));
-  gst_tag_list_free (taglist2);
+  gst_tag_list_unref (taglist2);
 
   /* APP1 */
   buf = gst_tag_list_to_exif_buffer_with_tiff_header (taglist);
@@ -1360,7 +1368,7 @@ do_exif_tag_serialization_deserialization (GstTagList * taglist)
   gst_buffer_unref (buf);
 
   fail_unless (gst_tag_list_is_equal (taglist, taglist2));
-  gst_tag_list_free (taglist2);
+  gst_tag_list_unref (taglist2);
 }
 
 static void
@@ -1372,7 +1380,7 @@ do_simple_exif_tag_serialization_deserialization (const gchar * gsttag,
   gst_tag_list_add_value (taglist, GST_TAG_MERGE_REPLACE, gsttag, value);
   do_exif_tag_serialization_deserialization (taglist);
 
-  gst_tag_list_free (taglist);
+  gst_tag_list_unref (taglist);
 }
 
 /*
@@ -1403,7 +1411,7 @@ GST_START_TEST (test_exif_multiple_tags)
 
   do_exif_tag_serialization_deserialization (taglist);
 
-  gst_tag_list_free (taglist);
+  gst_tag_list_unref (taglist);
 }
 
 GST_END_TEST;
@@ -1726,14 +1734,14 @@ GST_START_TEST (test_exif_tags_serialization_deserialization)
   do_simple_exif_tag_serialization_deserialization (GST_TAG_DATE_TIME, &value);
   g_value_unset (&value);
 
-  g_value_init (&value, GST_TYPE_BUFFER);
+  g_value_init (&value, GST_TYPE_SAMPLE);
   buf = gst_buffer_new_and_alloc (1024);
   gst_buffer_map (buf, &map, GST_MAP_WRITE);
   data = map.data;
   for (i = 0; i < 1024; i++)
     data[i] = i % 255;
   gst_buffer_unmap (buf, &map);
-  gst_value_set_buffer (&value, buf);
+  gst_value_take_sample (&value, gst_sample_new (buf, NULL, NULL, NULL));
   gst_buffer_unref (buf);
   do_simple_exif_tag_serialization_deserialization (GST_TAG_APPLICATION_DATA,
       &value);
@@ -1759,22 +1767,22 @@ GST_START_TEST (test_exif_tags_serialization_deserialization)
   taglist = gst_tag_list_new (GST_TAG_CAPTURING_FLASH_FIRED, FALSE,
       GST_TAG_CAPTURING_FLASH_MODE, "auto", NULL);
   do_exif_tag_serialization_deserialization (taglist);
-  gst_tag_list_free (taglist);
+  gst_tag_list_unref (taglist);
 
   taglist = gst_tag_list_new (GST_TAG_CAPTURING_FLASH_FIRED, TRUE,
       GST_TAG_CAPTURING_FLASH_MODE, "auto", NULL);
   do_exif_tag_serialization_deserialization (taglist);
-  gst_tag_list_free (taglist);
+  gst_tag_list_unref (taglist);
 
   taglist = gst_tag_list_new (GST_TAG_CAPTURING_FLASH_FIRED, FALSE,
       GST_TAG_CAPTURING_FLASH_MODE, "never", NULL);
   do_exif_tag_serialization_deserialization (taglist);
-  gst_tag_list_free (taglist);
+  gst_tag_list_unref (taglist);
 
   taglist = gst_tag_list_new (GST_TAG_CAPTURING_FLASH_FIRED, TRUE,
       GST_TAG_CAPTURING_FLASH_MODE, "always", NULL);
   do_exif_tag_serialization_deserialization (taglist);
-  gst_tag_list_free (taglist);
+  gst_tag_list_unref (taglist);
 }
 
 GST_END_TEST;
